/*;  geometrc.h {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.8 $$ {\rhead{\sl geometrc.h\quad\thepage}} */
/*;  $$Date: 1996/11/28 10:41:31 $$  */
/*;  $$Source: /home/shiiba/96prog/gtool/geometrc.h,v $$  */

#if !defined(geometrc_h)
#define geometrc_h

//typedef int Metric;	// ɸη
typedef double Metric;	// ɸη
typedef double ElvType;	// ɸͤη


class Point2d {		// 
 protected:
  Metric xd;		// xɸ
  Metric yd;		// yɸ
  long nd;		// μֹ
 public:
  Point2d() {}
  Point2d(Metric xVal, Metric yVal, long nVal = 0)
  {
    xd = xVal; yd = yVal; nd = nVal;
  }
  Point2d(const Point2d& a)
  {
    xd = a.xd; yd = a.yd; nd = a.nd;
  }
  Point2d& operator=(const Point2d& a)
  {
    if (this == &a) return *this;
    xd = a.xd; yd = a.yd; nd = a.nd;
    return *this;
  }
  Metric x() const { return xd; }
  Metric y() const { return yd; }
  long Number() { return nd; }
  void x(Metric xVal) { xd = xVal; }
  void y(Metric yVal) { yd = yVal; }
  void Number(long nVal) { nd = nVal; }
  void SetVal(Metric xVal, Metric yVal)
  {
    xd = xVal; yd = yVal;
  }
};

class Point3d : public Point2d { // 
 protected:
  ElvType zd;		// zɸ
 public:
  Point3d() {}
  Point3d(Metric xVal, Metric yVal, ElvType zVal, long nVal = 0)
  {
    xd = xVal; yd = yVal; zd = zVal; nd = nVal;
  }
  Point3d(const Point3d& a)
  {
    xd = a.xd; yd = a.yd; nd = a.nd; zd = a.zd;
  }
  Point3d& operator=(const Point3d& a)
  {
    if (this == &a) return *this;
    xd = a.xd; yd = a.yd; nd = a.nd; zd = a.zd;
    return *this;
  }
  Point3d& operator=(const Point2d& a)
  {
    xd = a.x();  yd = a.y();  zd = 0;
    return *this;
  }

  ElvType z() { return zd; }
  void z(ElvType zVal) { zd = zVal; }
  void SetVal(Metric xVal, Metric yVal, ElvType zVal)
  {
    xd = xVal; yd = yVal; zd = zVal;
  }
};

class Line3d;

class Line2d {		// ʬ
  
  Point2d p1d;
  Point2d p2d;
  long nd;
  
 public:
  Line2d() {}
  Line2d(Point2d p1Val, Point2d p2Val, long nVal = 0)
  {
    p1d = p1Val; p2d = p2Val; nd = nVal;
  }
  Line2d(const Line2d& a)
  {
    p1d = a.p1d; p2d = a.p2d; nd = a.nd;
  }
  Line2d(const Line3d& a);
  Line2d& operator=(const Line2d& a)
  {
    if (this == &a) return *this;
    p1d = a.p1d;  p2d = a.p2d; nd = a.nd;
    return *this;
  }
  Line2d& operator=(const Line3d& a);
  Point2d& p1() { return p1d; }
  Point2d& p2() { return p2d; }
  Point2d p1Val() const { return p1d; }
  Point2d p2Val() const { return p2d; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
};

class Line3d {		// ʬ
  Point3d p1d;
  Point3d p2d;
  long nd;
 public:
  Line3d() {}
  Line3d(Point3d p1Val, Point3d p2Val, long ndVal = 0)
  {
    p1d = p1Val; p2d = p2Val; nd = ndVal;
  }
  Line3d(const Line3d& a)
  {
    p1d = a.p1d; p2d = a.p2d; nd = a.nd;
  }
  Line3d(const Line2d& a)
  {
    p1d = a.p1Val(); p2d = a.p2Val();
  }
  Line3d& operator=(const Line3d& a)
  {
    if (this == &a) return *this;
    p1d = a.p1d;  p2d = a.p2d; nd = a.nd;
    return *this;
  }
  Line3d& operator=(const Line2d& a)
  {
    p1d = a.p1Val();  p2d = a.p2Val(); 
    return *this;
  }
  Point3d& p1() { return p1d; }
  Point3d& p2() { return p2d; }
  Point3d p1Val() const { return p1d; }
  Point3d p2Val() const { return p2d; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
};

inline
Line2d::Line2d(const Line3d& a)
{
  p1d = a.p1Val(); p2d = a.p2Val();
}

inline
Line2d& Line2d::operator=(const Line3d& a)
{
  p1d = a.p1Val();  p2d = a.p2Val();
  return *this;
}

class Polygon2d {	// ¿ѷ
 protected:
  Point2d *vertex;
  int size; // number of vertices
  long nd;
  
 public:
  
  Polygon2d(int sizeVal = 0);
  Polygon2d(const Polygon2d& a);
  Polygon2d& operator= (const Polygon2d& a);
  ~Polygon2d();
  
  void Resize(int sizeVal); // change size to sizeVal
  Line2d Side(int i);
  // return the side from vertex(i%size) to vertex((i+1)%size);
  Point2d& Vertex(int i) { return vertex[i % size]; }
  // return the i-th vertex. You can use a.Vertex(i) as left value.
  // Example: (a.Vertex(i)).x = 1; (a.Vertex(i)).y = 2;
  int Size(void) const { return size; }
  Metric Area2();
  int Inside(Point2d a);
  // find out whether the point is inside of the polygon or
  // not.
  // When the point is inside of the polygon, it returns 1.
  // When the point is on a side of the polygon, it returns
  // 0.
  // When the point is outside of the polygon, it returns -1.
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
};	

class Polygon3d {	// ¿ѷ
 protected:
  Point3d *vertex;
  int size; // number of vertices
  long nd;
  
 public:
  
  Polygon3d(int sizeVal = 0);
  Polygon3d(const Polygon3d& a);
  Polygon3d& operator= (const Polygon3d& a);
  ~Polygon3d();
  
  void Resize(int sizeVal); // change size to sizeVal
  Line3d Side(int i);
  // return the side from vertex(i) to vertex(i+1);
  Point3d& Vertex(int i) { return vertex[i % size]; }
  // return the i-th vertex. You can use a.Vertex(i) as left value.
  // Example: (a.Vertex(i)).x = 1; (a.Vertex(i)).y = 2;
  //          (a.Vertex(i)).z = 1
  int Size(void) const { return size; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
};	

class Triangle2d {	// Σѷ
 protected:
  Point2d v[3];
  long nd;
 public:
  Triangle2d() {}
  Triangle2d(Point2d p0, Point2d p1, Point2d p2, long nValue = 0) {
    SetData(p0, p1, p2, nValue);
  }
  Triangle2d(const Triangle2d& a);
  Triangle2d& operator= (const Triangle2d& a);
  Line2d Side(int i);
  Point2d& Vertex(int i) { return v[i % 3]; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
  Metric Area2();
  void SetData(Point2d p0, Point2d p1, Point2d p2, long nValue) {
    v[0] = p0;   v[1] = p1;   v[2] = p2;    nd = nValue;
  }
};

class Triangle3d {	// Σѷ
 protected:
  Point3d v[3];
  long nd;
 public:
  Triangle3d() {}
  Triangle3d(Point3d p0, Point3d p1, Point3d p2, long nValue = 0) {
    SetData(p0, p1, p2, nValue);
  }
  Triangle3d(const Triangle3d& a);
  Triangle3d& operator= (const Triangle3d& a);
  Line3d Side(int i);
  Point3d& Vertex(int i) { return v[i % 3]; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
  Metric HArea2();
  void SetData(Point3d p0, Point3d p1, Point3d p2, long nValue) {
    v[0] = p0;   v[1] = p1;   v[2] = p2;    nd = nValue;
  }
};

class QuadL2d {		// Σշ
 protected:
  Point2d vd[4];		// շΣĤĺ
  long nd;			// շμֹ
 public:
  QuadL2d() {}		// default constructor
  QuadL2d(const QuadL2d& q); // copy constructor
  QuadL2d(Point2d p0, Point2d p1, Point2d p2, Point2d p3, long nVal = 0)
			// construct from points
  {
    SetData(p0, p1, p2, p3, nVal);
  }
  QuadL2d(Point2d* pPtr, long nVal = 0); // construct from pointer
  QuadL2d& operator= (const QuadL2d& q);	// assignment operator
  Line2d Side(int i);
  Point2d& Vertex(int i) { return vd[i%4]; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
  Metric Area2();	// : շѤΣܤ
  int Inside(Point2d p); // Pշˤ뤫ˤ뤫
			 // :
			 //  1: շ
			 //  0: շդξ
			 // -1: շγ
  void SetData(Point2d p0, Point2d p1, Point2d p2, Point2d p3, long nVal)
  {
    vd[0] = p0;  vd[1] = p1;  vd[2] = p2;  vd[3] = p3;  nd = nVal;
  }
};

class QuadL3d {		// Σշ
 protected:
  Point3d vd[4];		// շΣĤĺ
  long nd;			// շμֹ
 public:
  QuadL3d() {}		// default constructor
  QuadL3d(const QuadL3d& q); // copy constructor
  QuadL3d(Point3d p0, Point3d p1, Point3d p2, Point3d p3, long nVal = 0)
			// construct from points
  {
    SetData(p0, p1, p2, p3, nVal);
  }
  QuadL3d(Point3d* pPtr, long nVal = 0); // construct from pointer
  QuadL3d& operator= (const QuadL3d& q);	// assignment operator
  Line3d Side(int i);
  Point3d& Vertex(int i) { return vd[i%4]; }
  long Number() const { return nd; }
  void Number(long nVal) { nd = nVal; }
  Metric HArea2();	// ʿ̤ƤշѤΣܤ֤ͤ
  int Inside(Point3d p);
  void SetData(Point3d p0, Point3d p1, Point3d p2, Point3d p3, long nVal)
  {
    vd[0] = p0;  vd[1] = p1;  vd[2] = p2;  vd[3] = p3;  nd = nVal;
  }
};

#endif

/*; end of file */
