/*;  grid.h {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl grid.h\quad\thepage}} */
/*;  $$Date: 1996/12/02 12:39:28 $$  */
/*;  $$Source: /home/shiiba/96prog/matrix/grid.h,v $$  */

#ifndef GRID_H
#define GRID_H

#include "boolean.h"
#include "stddef.h"
#include "limits.h"
// #include "simple.h"
#include "string.h"
#include "error.h"

//-----------------------------------
// Grid
//      A two-dimensional array class
//-----------------------------------

template <class D>
class Grid
{
 public:
  // constructors
  Grid();
  
  Grid
  (
   size_t rows,
   size_t cols
   );
  
  Grid
  (
   const Grid<D> & m
   );
  
  // destructors
  ~Grid();
  
  // assignment operator
  void operator =
  (
   const Grid<D> & m
   );
  
  void operator =
  (
   const D * array
   );
  
  // interrogation
  size_t GetRows() const;
  size_t GetCols() const;
  
  Boolean IsVector() const;
  Boolean IsRowVector() const;
  Boolean IsColVector() const;
  Boolean IsSquare() const;
  
  // retrieve elements
  D Get
  (
   size_t row,
   size_t col
   ) const;
  
  D & operator () (
    size_t row,
    size_t col
    );
  
  // internal function w/o exception check!
  D & Elem
  (
   size_t row,
   size_t col
   ) const;
       
  // apply a function to each element
  void Apply(
    D (* func)(const D & n)
    );
  
//  friend Grid<D> Apply(
//    const Grid<D> & m,
//    D (* func)(const D & n)
//    );
  
  friend Grid<D> Apply(
   const Grid<D> & m,
   D (* func)(const D & n)
   )
  {
      Grid<D> result(m);

      for (size_t i = 0; i < m.N; ++i)
          result.Data[i] = func(result.Data[i]);

      return result;
  }

  // fill matrix with specific value
  void Fill
  (
   const D & x
   );
  
  // create a row matrix
  Grid<D> VectorRow
  (
   size_t row
   );
  
  // create a column vector
  Grid<D> VectorCol
  (
   size_t col
   );
  
  // transpose a matrix
  Grid<D> Transpose();
  
  // change size (destroying contents)
  void Resize(
   size_t rows,
   size_t cols
   );
  
 protected:
  D * Data;
  size_t R;
  size_t C;
  size_t N;
  
  size_t Index
  (
   size_t row,
   size_t col
   ) const;
  
};

template <class D>
inline size_t Grid<D>::Index(
 size_t row,
 size_t col
 ) const
{
  return (row * C + col);
}

// interrogation
template <class D>
inline size_t Grid<D>::GetRows() const
{
  return R;
}

template <class D>
inline size_t Grid<D>::GetCols() const
{
  return C;
}

template <class D>
inline Boolean Grid<D>::IsVector() const
{
  if ((R == 1) || (C == 1))
    return YES;
  else
    return NO;
}

template <class D>
inline Boolean Grid<D>::IsRowVector() const
{
  if (C == 1)
    return YES;
  else
    return NO;
}

template <class D>
inline Boolean Grid<D>::IsColVector() const
{
  if (R == 1)
    return YES;
  else
    return NO;
}

template <class D>
inline Boolean Grid<D>::IsSquare() const
{
  if (R == C)
    return YES;
  else
    return NO;
}

template <class D>
inline D & Grid<D>::Elem
(
 size_t row,
 size_t col
 )
     const
{
  return Data[row * C + col];
}

#endif

/*; end of file */
