/*;  gridpoint.cc {\documentstyle{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.4 $$ {\rhead{\sl gridpoint.cc\quad\thepage}} */
/*;  $$Date: 1997/03/01 21:27:51 $$  */
/*;  $$Source: /home/sakakiba/96prog/slope/gridpoint.cc,v $$  */


#include <iostream>
#include <math.h>
#include <assert.h>
#include "gtool.h"
#include "gridpoint.h"
#include "error.h"

GridPoint2d::GridPoint2d()
{
  quadNum = 0;
  for (int i = 0; i < 4; i++)
    quadGroup[i] = NULL;
}

GridPoint2d::GridPoint2d(const GridPoint2d& gp)
{
  xd = gp.xd; yd = gp.yd; nd = gp.nd;
  quadNum = gp.quadNum;
  for (int i = 0; i < quadNum; i++)
    quadGroup[i] = (gp.quadGroup[i]);
}

GridPoint2d&
GridPoint2d::operator= (const GridPoint2d& gp)
{
  if (this == &gp) return *this;

  xd = gp.xd; yd = gp.yd; nd = gp.nd;
  quadNum = gp.quadNum;
  for (int i = 0; i < quadNum; i++)
    quadGroup[i] = (gp.quadGroup[i]);

  return *this;
}

void
GridPoint2d::AddQuad(QuadElmnt2d& gp)
{
  quadGroup[quadNum] = &gp;
  quadNum++;
}

void
GridPoint2d::Print(ostream& dst)
{
  dst << "No." << nd << "(" << xd << "," << yd << ") "
      << "total(" << quadNum << ") (";
  for (int i = 0; i < 4; i++)
    {
      if (quadGroup[i] != NULL)
	dst << quadGroup[i]->Number();
      else
	dst << "0";
      if (i < 3)  dst << ":" ;
      else  dst << ")" << endl;
    }

  return;
}

QuadElmnt2d::QuadElmnt2d(const QuadElmnt2d& q)
{
  row = q.row;  column = q.column;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
}

QuadElmnt2d&
QuadElmnt2d::operator= (const QuadElmnt2d& q)
{
  if (this == &q)
    return *this;
  for (int i = 0; i < 4; i++)
    vd[i] = q.vd[i];
  row = q.row;  column = q.column;
  return *this;
}

GridPoint2d*
QuadElmnt2d::Nearest(Point2d& p)
{
  double a, b, c, d;
  int vertexflag = 0;

  a = pow(fabs(vd[0]->x() - p.x()), 2) + pow(fabs(vd[0]->y() - p.y()), 2);
  b = pow(fabs(vd[1]->x() - p.x()), 2) + pow(fabs(vd[1]->y() - p.y()), 2);

  if (b <= a) vertexflag = 1;
 c = pow(fabs(vd[2]->x() - p.x()), 2) + pow(fabs(vd[2]->y() - p.y()), 2);
  if (vertexflag == 0) {
    if (c <= a) vertexflag = 2;
  }
  else {
    if (c <= b) vertexflag = 2;
  }
  d = pow(fabs(vd[3]->x() - p.x()), 2) + pow(fabs(vd[3]->y() - p.y()), 2);
  if (vertexflag == 0) {
    if (d <= a) vertexflag = 3;
  }
  else if (vertexflag == 1) {
    if (d <= b) vertexflag = 3;
  }
  else {
    if (d <= c) vertexflag = 3;
  }
  return vd[vertexflag];
}

Line2d
QuadElmnt2d::Side(int i)
{
  return Line2d(*(vd[i%4]), *(vd[(i+1)%4]));
}

Metric
QuadElmnt2d::Area2()
{
  return (::Area2(*(vd[0]), *(vd[1]), *(vd[2]))
    + ::Area2(*(vd[2]), *(vd[3]), *(vd[0])));
}

int
QuadElmnt2d::Inside(Point2d& p)
{
  int c1, c2, c3, c4;

  if ((c1 = Ccw(*(vd[0]), *(vd[1]), p)) == 0)
    return 0;

  if ((c2 = Ccw(*(vd[1]), *(vd[2]), p)) == 0)
    return 0;

  if ((c3 = Ccw(*(vd[2]), *(vd[3]), p)) == 0)
    return 0;

  if ((c4 = Ccw(*(vd[3]), *(vd[0]), p)) == 0)
    return 0;

  if (fabs(c1 + c2 + c3 + c4) == 4.)
    return 1;

  return -1;
}

void
QuadElmnt2d::Print(ostream& dst)
{
  dst << "<" << row << "," << column << "> ";
  for (int i = 0; i < 4; i++)
    dst << vd[i]->Number() << "(" << vd[i]->x() << ","
	<< vd[i]->y() << ") ";
  return;
}

GridPoint3d::GridPoint3d()
{
  cntnum = 0;
  area = 0;
  rivFlag = 0;
  edgeFlag = 0;
}

GridPoint3d::GridPoint3d(const GridPoint3d& gp)
{
  xd = gp.xd; yd = gp.yd; nd = gp.nd; zd = gp.zd;
  cntnum = gp.cntnum; rivFlag = gp.rivFlag;
  edgeFlag = gp.edgeFlag;
  area = gp.area;
  cntgpp = gp.cntgpp;
  
}

GridPoint3d&
GridPoint3d::operator= (const GridPoint3d& gp)
{
  if (this == &gp) return *this;

  xd = gp.xd; yd = gp.yd; nd = gp.nd; zd = gp.zd;
  cntnum = gp.cntnum; rivFlag = gp.rivFlag;
  edgeFlag = gp.edgeFlag;
  area = gp.area;
  cntgpp = gp.cntgpp;
  
  return *this;
}

void
GridPoint3d::ConnectGp(GridPoint3d* gpp)
{
  if (cntgpp.GetCount() > 0)
    {
      cntgpp.ToHead();
      while (1)
	{
	  if ((cntgpp.Get()) == gpp) return;
	  if (cntgpp.ToNext() == NO) break;
	}
    }
  
  cntgpp.Append(gpp);
  
  cntnum++;
  return;
}

void
GridPoint3d::CutGp(GridPoint3d* gpp)
{
  if (cntgpp.GetCount() == 0)
    return;
  else if (cntgpp.GetCount() > 0)
    {
      cntgpp.ToHead();
      while (1)
	{
	  GridPoint3d* tmpgp = cntgpp.Get();
	  if (tmpgp == gpp)
	    {
	      cntgpp.Delete();
	      cntnum--;
	      if (cntgpp.AtHead() == YES) return;
	    }
	  if (cntgpp.ToNext() == NO)
	    break;
	}
    }
  else
    {
      error();
      abort();
    }
  

  return;
}

void
//GridPoint3d::Print(ostream& dst)
GridPoint3d::Print(ostream&)
{
  return;
}

BasinPoint3d::BasinPoint3d()
{
  cntnum = 0;
  area = 0.;
  rsFlag = 0;
  cntbp = NULL;
}

BasinPoint3d::BasinPoint3d(const BasinPoint3d& bp)
{
  xd = bp.xd; yd = bp.yd; nd = bp.nd; zd = bp.zd;
  rsFlag = bp.rsFlag;
  area = bp.area;
  inList.Erase(); inList = bp.inList; 
  outList.Erase(); outList = bp.outList;

  delete [] cntbp;

  cntnum = bp.cntnum;
  cntbp = new BasinPoint3d*[cntnum];
  
  for (int i = 0; i < cntnum; i++)
    {
      *(cntbp + i) = bp.NeighborBp(i);
    }
}

BasinPoint3d::~BasinPoint3d()
{
  delete [] cntbp;
}

BasinPoint3d&
BasinPoint3d::operator= (const BasinPoint3d& bp)
{
  if (&bp == this) return *this;

  xd = bp.xd; yd = bp.yd; nd = bp.nd; zd = bp.zd;
  rsFlag = bp.rsFlag;
  area = bp.area;
  inList.Erase(); inList = bp.inList; 
  outList.Erase(); outList = bp.outList;

  delete [] cntbp;

  cntnum = bp.cntnum;
  cntbp = new BasinPoint3d*[cntnum];
 
  for (int i = 0; i < cntnum; i++)
    {
      *(cntbp + i) = bp.NeighborBp(i);
    }
  return *this;
}

void
BasinPoint3d::New(int sizeVal)
{
  if (sizeVal <= 0) { error(); abort(); }
  cntbp =  new BasinPoint3d*[sizeVal];
  assert(cntbp != NULL);
  return;
}

void
BasinPoint3d::ConnectBp(int num, BasinPoint3d* bp)
{
  *(cntbp + num) = bp;
  return;
}

void
BasinPoint3d::CutBp(BasinPoint3d* bp)
{
  if (cntnum == 0) return;
  else if (cntnum < 0) { error(); abort(); }

  int checkval = -1;
  for (int i = 0; i < cntnum; i++)
    {
      if (*(cntbp + i) == bp)
	{ 
	  checkval = i; 
	  break; 
	}
    }

  if (checkval == -1) return;
  else if (checkval == 0 && cntnum == 1) { cntnum = -1; return; }
  else if (checkval < cntnum - 1)
    {
      for (int j = checkval; j < cntnum - 1; j++)
	{
	  *(cntbp + j) = *(cntbp + j + 1);
	}
    }

  cntnum--;
  return;
}

void
BasinPoint3d::Print(ostream& dst)
{
  dst << setw(3) << nd 
      << setw(4) << xd << setw(4) << yd << setw(4) << zd
      << setw(8) << area << setw(3) << rsFlag << setw(3) << cntnum;

  for (int i = 0; i < cntnum; i++)
    {
      dst << setw(3) << (*(cntbp + i))->Number();
    }

  dst << endl;

  return;
}

/*; end of file */
