/* slist.cc {\documentstyle{jsrc2latex}\rhead{\sl slist.cc\quad\thepage}
   \font\tenrm=cmtt10}
{\ \begin{verbatim}
$RCSfile: slist.cc,v $  $Revision: 1.1 $ 
$Date: 1996/09/05 10:43:15 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode

}% end of latex-mode }
*/

#ifndef slist_cc
#define slist_cc

#include <iostream>
#include "boolean.h"
#include "slist.h"

using namespace std;   // added on Oct. 11, 2007

// implement utility functions
template <class T>
    void SListDynamic<T>::DeepCopy
        (
        const SListDynamic<T> & slst
        )
        {
        if (slst.Count == 0)
            return;

        Node * n = slst.Head;

        do  {
            Append(n->Data);
            n = n->Next;
            }
        while (n != NULL);

        Work = Head;
        Shallow  = NO;
        Circular = slst.Circular;
        }

template <class T>
    inline void SListDynamic<T>::ShallowCopy
        (
        const SListDynamic<T> & slst
        )
        {
        Head  = slst.Head;
        Tail  = slst.Tail;
        Work  = slst.Work;
        Count = slst.Count;
        Shallow  = YES;
        Circular = slst.Circular;
        }

template <class T>
    inline void SListDynamic<T>::SetNull()
        {
        Head  = NULL;
        Tail  = NULL;
        Work  = NULL;
        Count = 0;
        }

// constructors
template <class T>
    inline SListDynamic<T>::SListDynamic
        (
        Boolean circ
        )
        {
        SetNull();
        Shallow  = NO;
        Circular = circ;
        }

// copy constructor
template <class T>
    SListDynamic<T>::SListDynamic
        (
        const SListDynamic<T> & slst,
        Boolean shallow
        )
        {
        if (shallow)
            ShallowCopy(slst);
        else
            {
            SetNull();
            DeepCopy(slst);
            }
        }

// construct from array
template <class T>
    SListDynamic<T>::SListDynamic
        (
        const T * array,
        size_t no
        )
        {
        SetNull();

        Circular = NO;
        Shallow  = NO;

        if ((array == NULL) || (no == 0)) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        const T * aptr = array;

        for (size_t n = 0; n < no; ++n)
            {
            Append(*aptr);
            ++aptr;
            }
        }

// destructor
template <class T>
    inline SListDynamic<T>::~SListDynamic()
        {
        Erase();
        }

// assignment operator (deep copy)
template <class T>
    void SListDynamic<T>::operator =
        (
        const SListDynamic<T> & slst
        )
        {
        Erase();
        DeepCopy(slst);
        }

// conversion to array operator
template <class T>
    SListDynamic<T>::operator T * ()
        {
        if (Count == 0)
            return NULL;

        T * result = new T [Count];

        if (result == NULL) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        Node * temp = Head;
        T    * aptr = result;

        while (temp != NULL)
            {
            *aptr = temp->Data;
            ++aptr;
            temp = temp->Next;
            }

        // note: caller must delete this array!
        return result;
        }

// append new item
template <class T>
    void SListDynamic<T>::Append
        (
        const T & item
        )
        {
        Node * n = new Node;

        if (n == NULL) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        n->Data = item;
        n->Next = NULL;

        if (Tail == NULL)
            {
            Head = n;
            Tail = n;
            Work = n;
            }
        else
            {
            Tail->Next = n;
            Tail = n;
            }

        ++Count;
        }

// append an array of new items
template <class T>
    void SListDynamic<T>::Append
        (
        const T * array,
        size_t no
        )
        {
		if ((array == NULL) || (no == 0)) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        const T * aptr = array;

        for (size_t n = 0; n < no; ++n)
            {
            Append(*aptr);
            ++aptr;
            }
        }

template <class T>
    void SListDynamic<T>::Append
        (
        const SListDynamic<T> & slst
        )
        {
        const Node * n = slst.Head;

        while (n != NULL)
            {
            Append(n->Data);
            n = n->Next;
            }
        }

// remove current item
// ιܤդɲäޤ
template <class T>
void SListDynamic<T>::Remove() {
	if (Work == NULL) {
		cerr << "Errot: at " << __FILE__ << ": " << __LINE__ << endl;
		exit(1);
	}
	
	Node * RemoveNode = Work;
	if (RemoveNode == Head) {
		if (RemoveNode == Tail) {
			if (!Shallow) delete RemoveNode;
			Head = NULL;   Tail = NULL;   Work = NULL;  Count = 0;
			return;
		}
		Head = RemoveNode->Next;
		if (!Shallow) delete RemoveNode;
		Work = Head->Next;
		Count -= 1;
		return;
	}
	else {
		Work = Head;
		while (Work->Next != RemoveNode)
			Work = Work->Next;

		if (RemoveNode == Tail) {
			Work->Next = NULL;
			Tail = Work;
		}
		else {
			Work->Next = RemoveNode->Next;
		}
		if (!Shallow) delete RemoveNode;
		Count -= 1;
		return;
	}
}		
			
// remove all items from the list
template <class T>
    void SListDynamic<T>::Erase()
        {
        if (!Shallow)
            {
            Work = Head;

            while (Work != NULL)
                {
                Head = Work->Next;
                delete Work;
                Work = Head;
                }
            }

        SetNull();
        Shallow = NO;
        }

// get current item
template <class T>
    T SListDynamic<T>::Get()
        {
		if (Count == 0) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}
				
        if (Work == NULL) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        return Work->Data;
        }

// TRUE if current item is last item
template <class T>
    inline Boolean SListDynamic<T>::AtTail()
        {
        if (Work == Tail)
            return YES;
        else
            return NO;
        }

// set current item to head of list
template <class T>
    inline void SListDynamic<T>::ToHead()
        {
        Work = Head;
        }

// move to next item in list
template <class T>
    Boolean SListDynamic<T>::ToNext()
        {
		if (Count == 0) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        if (Work == NULL) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        if (Work == Tail)
            {
            if (Circular)
                Work = Head;
            else
                return NO;
            }
        else
            Work = Work->Next;

        return YES;
        }

// interrogate list
template <class T>
    inline size_t SListDynamic<T>::GetCount()
        {
        return Count;
        }

template <class T>
    inline Boolean SListDynamic<T>::IsShallow()
        {
        return Shallow;
        }

template <class T>
    inline Boolean SListDynamic<T>::IsCircular()
        {
        return Circular;
        }

// implement utility functions
template <class T, size_t Limit>
    void SList<T,Limit>::DeepCopy
        (
        const SList<T,Limit> & slst
        )
        {
        Count = slst.Count;
        Work  = slst.Work;

        for (size_t n = 0; n < Count; ++n)
            Data[n] = slst.Data[n];

        Circular = slst.Circular;
        }

// remove all items from the list
template <class T, size_t Limit>
    inline void SList<T,Limit>::Erase()
        {
        Count = 0;
        Work  = 0;
        }

// constructors
template <class T, size_t Limit>
    inline SList<T,Limit>::SList
        (
        Boolean circ
        )
        {
        Count    = 0;
        Work     = 0;
        Circular = circ;
        }

// copy constructor
template <class T, size_t Limit>
    inline SList<T,Limit>::SList
        (
        const SList<T,Limit> & slst
        )
        {
        DeepCopy(slst);
        }

// construct from array
template <class T, size_t Limit>
    SList<T,Limit>::SList
        (
        const T * array,
        size_t no
        )
        {
		if ((array == NULL) || (no == 0)) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        Count = 0;
        Work  = 0;
        Circular = NO;

        const T * aptr = array;

        for (size_t n = 0; n < no; ++n)
            {
            Append(*aptr);
            ++aptr;
            }
        }

// destructor
template <class T, size_t Limit>
    inline SList<T,Limit>::~SList()
        {
        Erase();
        }

// assignment operator (deep copy)
template <class T, size_t Limit>
    inline void SList<T,Limit>::operator =
        (
        const SList<T,Limit> & slst
        )
        {
        Erase();
        DeepCopy(slst);
        }

// conversion to array operator
template <class T, size_t Limit>
    SList<T,Limit>::operator T * ()
        {
        if (Count == 0)
            return NULL;

        T * result = new T [Count];

        if (result == NULL) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        for (size_t n = 0; n < Count; ++n)
            result[n] = Data[n];

        // note: caller must delete this array!
        return result;
        }

// append new item
template <class T, size_t Limit>
    void SList<T,Limit>::Append
        (
        const T & item
        )
        {
		if (Count == Limit) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        Data[Count] = item;

        ++Count;
        }

// append an array of new items
template <class T, size_t Limit>
    void SList<T,Limit>::Append
        (
        const T * array,
        size_t no
        )
        {
		if ((array == NULL) || (no == 0)) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        const T * aptr = array;

        for (size_t n = 0; n < no; ++n)
            {
            Append(*aptr);
            ++aptr;
            }
        }

template <class T, size_t Limit>
    void SList<T,Limit>::Append
        (
        const SList<T,Limit> & slst
        )
        {
        for (size_t n = 0; n < slst.Count; ++n)
            Append(slst.Data[n]);
        }

// get current item
template <class T, size_t Limit>
    T SList<T,Limit>::Get()
        {
		if (Count == 0) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

		if (Work == Count) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        return Data[Work];
        }

// TRUE if current item is last item
template <class T, size_t Limit>
    inline Boolean SList<T,Limit>::AtTail()
        {
        if (Work == Count)
            return YES;
        else
            return NO;
        }

// set current item to head of list
template <class T, size_t Limit>
    inline void SList<T,Limit>::ToHead()
        {
        Work = 0;
        }

// move to next item in list
template <class T, size_t Limit>
    Boolean SList<T,Limit>::ToNext()
        {
		if (Count == 0) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

		if (Work == Count) {
			cerr << "Error at " << __FILE__ << ": " << __LINE__ << endl;
			exit(1);
		}

        ++Work;

        if (Work == Count)
            {
            if (Circular)
                Work = 0;
            else
                return NO;
            }

        return YES;
        }

// interrogate list
template <class T, size_t Limit>
    inline size_t SList<T,Limit>::GetCount()
        {
        return Count;
        }

template <class T, size_t Limit>
    inline size_t SList<T,Limit>::GetLimit()
        {
        return Limit;
        }

template <class T, size_t Limit>
    inline Boolean SList<T,Limit>::IsCircular()
        {
        return Circular;
        }

#endif
 
/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */

